;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;This script used to calculate and plot the zonal mean variables from CAM;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;The terms zonal and meridional are used to describe directions on a globe;;;;;;;
;;;;;;;Zonal means "along a latitudinal circle" or "in the west–east direction";;;;;;;;
;;;;;;;while meridional means "along a longitudinal circle" (a.k.a. meridian) ;;;;;;;;;
;;;;;;;or "in the north–south direction" [2];;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_code.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/gsn_csm.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/csm/contributed.ncl"
load "$NCARG_ROOT/lib/ncarg/nclscripts/esmf/ESMF_regridding.ncl"

begin

  l_plot_cntl       = True  ; if True, plot the CNTL origianl field on the leftmost panel
  plot_leftstr      = True    ; if True, the lefadd igure captions as left string
  plot_sig_test     = True

  datadir       = "../../clim_evaluation/fig_data/"

  ;;;;;;;;;;select the regions to calculate the rmse;;;;;;;;;;;;;
  Groups      = (/"ERA5","CLIM30","CLIM30ML"/)
  ngrps       = dimsizes(Groups)
  LabList     = (/"ERA5","CLIM - ERA5","ML(CLIM) - ERA5"/)
  yst         = 1979
  yed         = 2014

  PltNums     = (/"(a)", "(b)", "(c)"/)

  VarList     = (/"PSL"/)
  VarName     = "PSL"
  VarUnit     = "hPa"
  VarStr      = "Mean sea level pressure (PSL, hPa)"
  VarSt1      = "Bias in PSL (hPa)"
  nvars       = dimsizes(VarList)
  rstr        = "Annual mean" ;VarName+ "(" + VarUnit+")" 
  do igp = 1, ngrps-1,1

   model = Groups(igp)
   obs   = Groups(0)

   do iv = 0, nvars-1,1

    fl0  = datadir+obs+"_"+VarList(iv)+"_"+yst+"-"+yed+".nc"
    f0   = addfile(fl0, "r")

    fl1  = datadir+model+"_"+VarList(iv)+"_"+yst+"-"+yed+".nc"
    f1   = addfile(fl1, "r")
   
    if(iv.eq.0) then 
      lat = f1->lat
      lon = f1->lon 
      varm = new((/nvars,dimsizes(lat),dimsizes(lon)/),typeof(f1->$VarList(iv)$))
      varm!0   = "var"
      varm!1   = "lat"
      varm!2   = "lon"
      varm&lat = lat 
      varm&lon = lon 
      varo     = varm 
      vsdm     = varm 
      vsdo     = varm
    end if 
    varo(iv,:,:)  = (/f0->$VarList(iv)$/)
    varm(iv,:,:)  = (/f1->$VarList(iv)$/)
    vsdo(iv,:,:)  = (/f0->variance/)
    vsdm(iv,:,:)  = (/f1->variance/)
   end do 
   printVarSummary(varo)
   printVarSummary(varm)

   vard = varm
   vard = varm - varo 

  ;calculate the global/regional mean values
   latrg = f1->lat
   rad   = 4.*atan(1.)/180.    ;
   wgty  = cos(latrg*rad)      ; cosine weights

   sigr  = 0.05                                   ; critical sig lvl for r
   xAve  = varm(0,:,:)
   yAve  = varo(0,:,:)
   xVar  = vsdm(0,:,:) 
   yVar  = vsdo(0,:,:)
   xN    = yed - yst +1
   yN    = yed - yst +1 

   iflag = False                        ; population variance similar
   prob  = xAve
   prob  = (/ttest(xAve,xVar,xN, yAve,yVar,yN, iflag, False)/)
   prob  = where(prob.lt.sigr,prob,prob@_FillValue )
   delete([/xAve,yAve,xVar,yVar,sigr,xN,yN,iflag/])

   ;----------------------------------------------------------------------
   ; Plotting section
   ;----------------------------------------------------------------------

   if(igp.eq.1)then

    ;;;;;;;open a wks for plot;;;;;;;;;;;;;;;;;;;
    wtype = "pdf"
   ;wtype@wkPaperSize  = "A4"
   ;wtype@wkOrientation   = "landscape" ;;pictures displayed along the length
 
    plotname = "fig_2d_map_clim_bias_psl"
    wks1 = gsn_open_wks(wtype,plotname)
    print(" ")
    print("........................... ")
    print("plot : " + plotname+".pdf")
    print("........................... ")
    plot     = new(ngrps,"graphic")
    ;;colors for the original plot
    omap0      = read_colormap_file("prcp_3")
    orgcolors := (/omap0(3,:),omap0(4,:),omap0(5,:),omap0(6,:), omap0(7,:), \
                   omap0(10,:),omap0(11,:),omap0(12,:),omap0(13,:), \
                   omap0(14,:),omap0(15,:), omap0(16,:), omap0(17,:)/)
    ;colors for the difference plot
    gsn_merge_colormaps(wks1,"temp_diff_18lev","WhViBlGrYeOrReWh")
    cncolors   = (/72,62,55,5,7,10,0,120,14,16,91,85,77/)
    orglev    := (/970,975,985,990,995,1000,1002,1005,1010,1015,1020,1025/)*1.0
    cnlevels  := (/-6.5,-5.5,-4.5,-3.5,-2.5,-1.5,1.5,2.5,3.5,4.5,5.5,6.5/)
   end if 

   ;;;set up the plot information;;;;
   ;load "./var_share_colorbar.ncl"

;---Resources to share between both plots
   res                = True     ; Plot modes desired.
   res@gsnDraw        = False
   res@gsnFrame       = False
   res@gsnMaximize    = False     ; Maximize plot
   res@tmXTOn         = False
   res@tmYROn         = False

   BorderThick = 1.0
   res@tmBorderThicknessF    = BorderThick
   res@tmXBMajorThicknessF   = BorderThick
   res@tmXBMinorThicknessF   = BorderThick*0.5
   res@tmYLMajorThicknessF   = BorderThick
   res@tmYLMinorThicknessF   = BorderThick*0.5
   res@tmXTMajorThicknessF   = BorderThick
   res@tmXTMinorThicknessF   = BorderThick*0.5
   res@tmYRMajorThicknessF   = BorderThick
   res@tmYRMinorThicknessF   = BorderThick*0.5

   FontHeightF = 0.022
   res@tiMainFontHeightF    = FontHeightF
   res@tmYLLabelFontHeightF = FontHeightF
   res@tmXBLabelFontHeightF = FontHeightF

   mpres     = res
   resx      = res
   resx1     = res

   res@gsnStringFontHeightF  = FontHeightF*2.0
   res@lbLabelFontHeightF    = FontHeightF*0.6
   res@lbTitleFontHeightF    = FontHeightF*0.8

   res@cnFillOn              = True     ; color plot desired
  ;res@cnFillPalette         = "ViBlGrWhYeOrRe";"testcmap" ;"NCV_jaisnd";"temp_19lev"
   res@cnLinesOn             = False    ; turn off contour lines
   res@cnLineLabelsOn        = False    ; turn off contour labels
  ;res@cnFillMode            = "RasterFill"   ; turn raster on
   res@cnMissingValFillColor = "White"

   res@lbLabelBarOn             = False; True   ; Will turn on in panel later
   res@pmLabelBarParallelPosF   = 0.50 ;61
   res@pmLabelBarOrthogonalPosF = 0.20
  ;res@lbOrientation            = "Horizonal" ; "vertical"  ; vertical label bar
   res@pmLabelBarHeightF        = 0.12 ;0.60
   res@pmLabelBarWidthF         = 0.65 ;0.06
  ;res@lbTitleString            = "" ;Varunt;Varstr+ " ("+Varunt+")";
  ;res@lbTitlePosition          = "Right"         ; title location
  ;res@lbTitleDirection         = "Across"        ; letter angle
  ;res@lbTitleAngleF            = 90.          ; title angle

   res@gsnCenterString    = ""
   res@gsnLeftString      = ""
   res@gsnRightString     = "" 
   res@tiMainString       = "" 

;---Control appearance of map.
   mpres@gsnMajorLatSpacing     = 30
   mpres@gsnMajorLonSpacing     = 45
   mpres@gsnMinorLatSpacing     = 15
   mpres@gsnMinorLonSpacing     = 15
  ;mpres@mpProjection           = "Robinson"       ; choose projection
   mpres@mpLabelsOn             = False
   mpres@mpPerimOn              = False
   mpres@mpFillOn               = False
   mpres@mpGeophysicalLineColor = "Black"  ; color of cont. outlines
   mpres@mpGeophysicalLineThicknessF = 0.1          ; thickness of outlines
  ;mpres@mpOutlineOn            = True
  ;mpres@mpOutlineDrawOrder     = "PostDraw"
  ;mpres@mpFillDrawOrder        = "Predraw"
  ;mpres@mpGridAndLimbOn        = True
  ;mpres@mpGridLatSpacingF      = 90.0
  ;mpres@mpGridLonSpacingF      = 180.0
  ;mpres@mpGridLineThicknessF   = 1.0
  ;mpres@mpGridAndLimbDrawOrder="PreDraw"
  ;mpres@mpMinLonF              = 0  ;-180.
  ;mpres@mpMaxLonF              = 360;180.
   mpres@mpMinLatF              = -70
   mpres@mpMaxLatF              =  70
   mpres@mpCenterLonF           = 180.0
   mpres@tmYLLabelsOn           = True
   mpres@tmYLOn                 = True
   mpres@tmYROn                 = False

   plot(0)   = gsn_csm_map(wks1,mpres)
   plot(igp) = gsn_csm_map(wks1,mpres)

;---Resources for plotting original data
   res1=res
   res1@gsnAddCyclic                = True
   res1@cnLevelSelectionMode        = "ExplicitLevels"  
   res1@cnLevels                    = orglev
   res1@cnFillColors                = orgcolors
   res1@cnFillPalette               = "NCV_jaisnd" ;"WhBlGrYeRe" ;"BlAqGrYeOrReVi200"
   res1@gsnLeftString               = PltNums(0) + "  " +LabList(0) 
   res1@gsnLeftStringOrthogonalPosF = 0.04
  ;res1@gsnLeftStringParallelPosF  = 0.2
   res1@gsnRightString              = rstr 
   res1@gsnRightStringOrthogonalPosF= 0.055
   res1@cnLinesOn                   = False ;plot_contour  
   res1@cnLineLabelsOn              = False  
   res1@cnInfoLabelOn               = False
   res1@cnLineLabelDensityF         = 2.0
   res1@cnLineLabelFontHeightF      = 0.020
   res1@cnLineLabelBackgroundColor  = -1
   res1@gsnContourLineThicknessesScale = 1.0
   res1@cnLineThicknessF            = 0.001
   res1@cnLineColor                 = "Black"
   res1@gsnContourZeroLineThicknessF= 0.2
   res1@gsnContourNegLineDashPattern= 0
  ;res1@sfXArray     = lon
  ;res1@sfYArray     = lat
   plot_org1  = gsn_csm_contour(wks1,varo(0,:,:),res1)
   overlay(plot(0) ,plot_org1)

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    res2=res
    res2@gsnAddCyclic            = True
    res2@cnLevelSelectionMode     = "ExplicitLevels"  
    res2@cnFillColors             = cncolors
    res2@cnLevels                 = cnlevels
    res2@cnLinesOn                = False    ; turn off contour lines
    res2@cnLineLabelsOn           = False   ; turn off contour labels
    res2@cnInfoLabelOn            = False
    res2@gsnLeftString            = PltNums(igp)+"  " + LabList(igp)
    res2@gsnLeftStringOrthogonalPosF = 0.04
   ;res2@gsnLeftStringParallelPosF   = 0.2 
    res2@gsnRightString     = rstr 
    res2@gsnRightStringOrthogonalPosF   = 0.055
    plot_rgd1 = gsn_csm_contour(wks1,vard(0,:,:),res2)  
    overlay(plot(igp),plot_rgd1)


    if(plot_sig_test) then
      res21 = res2
      res21@gsnLeftString                = ""
      res21@gsnRightString               = ""
      res21@gsnCenterString              = ""
      res21@cnMissingValFillColor        = "Transparent"
      res21@cnFillOn                     = True
      res21@cnLinesOn                    = False
      res21@cnLineLabelsOn               = False       ; turn off contour labels
      res21@cnInfoLabelOn                = False
      res21@cnMonoFillColor              = True        ; default color is fground [black]
      res21@cnMonoFillPattern            = True
      res21@cnMonoFillScale              = True
      res21@cnFillPattern                = 17
      res21@cnFillScaleF                 = 0.5
      res21@cnFillDotSizeF               = 0.002
      res21@lbLabelBarOn                 = False            
      plot_cont2 = gsn_csm_contour(wks1,prob,res21)
      overlay(plot(igp),plot_cont2)
      delete([/res21/])
    end if

    delete([/res,res2,resx,mpres/])

  end do ; end of experiment group loop

;---Specify y positions of each row of plots.
  nrows       = 2
  plot_height = 1./tofloat(nrows)

  ypos1  = 0.96     ; top row
  ypos2  = 0.64     ; middle row
  dist_from_title_to_plot = 0.013
  ytitl1 = ypos1 + dist_from_title_to_plot
  ytitl2 = ypos2 + dist_from_title_to_plot

  pres                          = True 
; pres@gsnPanelDebug            = True      ; prints debug info
  pres@gsnFrame                 = False
  pres@gsnMaximize              = False
  pres@gsnPanelMainFontHeightF  = 0.015
  pres@gsnPanelFigureStrings    = "" 
  pres@gsnPanelFigureStringsFontHeightF  = 0.01

;---Top row
  pres1 = pres
; pres1@vpXF                     = 0.2         
; pres1@vpWidthF                 = 0.6	    
; pres1@vpYF                     = 0.83
; pres1@vpHeightF                = 0.465   
  pres1@gsnPanelLabelBar         = True 
  pres1@pmLabelBarParallelPosF   = 0.02
  pres1@pmLabelBarOrthogonalPosF = -0.02
  pres1@pmLabelBarHeightF        = 0.04 ;0.60
  pres1@pmLabelBarWidthF         = 0.44 ;0.06
  pres1@lbLabelFontHeightF       = FontHeightF*0.5
  pres1@lbTitleFontHeightF       = FontHeightF*0.55
  pres1@lbTitleString            = VarStr
  pres1@gsnPanelTop              = 1.0    
  pres1@gsnPanelBottom           = 0.5 
  pres1@gsnPanelYF              := (/ypos1/)    
  pres1@gsnPanelMainString       = "" 
  pres1@gsnPanelMainPosYF        = ytitl1
  gsn_panel(wks1,plot(0),(/1,2/),pres1)

;---Bottom row
  pres2 = pres 
; pres2@vpXF                     = 0.15   
; pres2@vpYF                     = 0.3     
; pres2@vpWidthF                 = 0.7
; pres2@vpHeightF                = 0.18
  pres2@gsnPanelLabelBar         = True
  pres2@pmLabelBarParallelPosF   = 0.02
  pres2@pmLabelBarOrthogonalPosF = -0.02
  pres2@pmLabelBarHeightF        = 0.04 ;0.60
  pres2@pmLabelBarWidthF         = 0.80 ;0.06
  pres2@lbLabelFontHeightF       = FontHeightF*0.5
  pres2@lbTitleFontHeightF       = FontHeightF*0.55
  pres2@lbTitleString            = VarSt1 
  pres2@gsnPanelTop              = 0.5
  pres2@gsnPanelBottom           = 0.0
  pres2@gsnPanelYF              := (/ypos2,ypos2/)
  pres2@gsnPanelMainString       = ""
  pres2@gsnPanelMainPosYF        = ytitl2    ; Y position of title
  gsn_panel(wks1,plot(1:),(/1,2/),pres2)

  frame(wks1)


end
